# -*- coding: utf-8 -*-

"""Main Controller"""

from tg import expose, flash, require, url, request, redirect, app_globals, \
               tmpl_context
from pylons.i18n import ugettext as _, lazy_ugettext as l_
from repoze.what import predicates

# TODO: Remove?
from catwalk.tg2 import Catwalk

from luci.controllers.decorators import *
from luci.lib.base import BaseController
from luci.lib.helpers import relativeUrlList2Str

from luci.controllers.error import ErrorController
# TODO: Remove? See 'luci.controllers.secure'.
from luci.controllers.secure import SecureController
from luci.controllers.async import AsyncController

from luci.model import DBSession, metadata
from luci import model

from luci.widgets.add_system_form import create_add_system_form
from luci.widgets.add_user_form import create_add_user_form
from luci.widgets.manage_systems_form import create_manage_systems_form

__all__ = ['RootController']


# Imports into module's namespace.
scheme = app_globals.scheme


@mountSubControllers(scheme.mounts)
class RootController(BaseController):
    """
    The root controller for the Luci application.

    All the other controllers and WSGI applications should be mounted on this
    controller. For example::

        panel = ControlPanelController()
        another_app = AnotherWSGIApplication()

    Keep in mind that WSGI applications shouldn't be mounted directly: They
    must be wrapped around with :class:`tg.controllers.WSGIAppController`.

    """

    def __call__(self, environ, start_response):
        """Set some defaults within 'tmpl_context' and invoke the Controller."""
        # Remembers, which title part has been already used.
        tmpl_context.already_used = {}
        # Vars for title parts adding and remembering appropriate arguments.
        tmpl_context.title_list = []
        tmpl_context.title_args = {}
        return BaseController.__call__(self, environ, start_response)


    # SUBCONTROLLERS

    error = ErrorController()
    async = AsyncController()

    # TODO: Remove? See 'luci.controllers.secure'.
    secc = SecureController()

    # TODO: Remove?
    admin = Catwalk(model, DBSession)


    # METHODS OF THE ROOT CONTROLLER

    # Homebase pages.

    @expose()
    # Uncomment this (or replace with more suitable decorator/predicate)
    # to apply access control.
    #@require(predicates.not_anonymous())
    def index(self):
        """Handle the front-page."""
        redirect ('/homebase')

    @expose('luci.templates.homebase')
    # Uncomment this (or replace with more suitable decorator/predicate)
    # to apply access control.
    #@require(predicates.not_anonymous())
    def homebase(self):
        """Handle the front-page."""

        base_url_str = relativeUrlList2Str(scheme.APP_PREFIX, scheme.CLUSTERS)

        return dict(page='homebase', homebasepage='homebasepage',
                    base_url=base_url_str)

    # -- Start of section of methods that might be moved somewhere
    # into more specific subcontrollers.

    @expose('luci.templates.homebase')
    # Uncomment this (or replace with more suitable decorator/predicate)
    # to apply access control.
    #@require(predicates.not_anonymous())
    def addsystem(self):
        tmpl_context.form = create_add_system_form
        return dict(page='homebase', homebasepage='addsystem')

    @expose('luci.templates.homebase')
    # Uncomment this (or replace with more suitable decorator/predicate)
    # to apply access control.
    #@require(predicates.not_anonymous())
    def adduser(self):
        tmpl_context.form = create_add_user_form
        return dict(page='homebase', homebasepage='adduser')

    @expose('luci.templates.homebase')
    # Uncomment this (or replace with more suitable decorator/predicate)
    # to apply access control.
    #@require(predicates.not_anonymous())
    def managesystems(self):
        tmpl_context.form = create_manage_systems_form
        return dict(page='homebase', homebasepage='managesystems')

    # -- End of section of methods that might be moved somewhere
    # into more specific subcontrollers.

    @expose('luci.templates.about')
    def about(self):
        """Handle the 'about' page."""
        return dict(page='about')

    @expose('luci.templates.storage')
    def storage(self,storagepage='systemlist'):
        return dict(page='storage', storagepage=storagepage)

    @expose('luci.templates.login')
    def login(self, came_from=url('/')):
        """Start the user login."""
        login_counter = request.environ['repoze.who.logins']
        if login_counter > 0:
            flash(_('Wrong credentials'), 'warning')
        return dict(page='login', login_counter=str(login_counter),
                    came_from=came_from)


    # AUTHENTICATION (LOGIN) METHODS OF THE ROOT CONTROLLER

    @expose()
    def post_login(self, came_from=url('/')):
        """
        Redirect the user to the initially requested page on successful
        authentication or redirect her back to the login page if login failed.

        """
        if not request.identity:
            login_counter = request.environ['repoze.who.logins'] + 1
            redirect(url('/login', came_from=came_from, __logins=login_counter))
        userid = request.identity['repoze.who.userid']
        flash(_('Welcome back, %s!') % userid)
        redirect(came_from)

    @expose()
    def post_logout(self, came_from=url('/')):
        """
        Redirect the user to the initially requested page on logout and say
        goodbye as well.

        """
        flash(_('We hope to see you soon!'))
        redirect(came_from)


    # TODO: Automatically generated/experimental methods? Perhaps cleaning
    # the methods below needed.

    @expose('luci.templates.authentication')
    def auth(self):
        """Display some information about auth* on this application."""
        return dict(page='auth')

    @expose('luci.templates.index')
    @require(predicates.has_permission('manage', msg=l_('Only for managers')))
    def manage_permission_only(self, **kw):
        """Illustrate how a page for managers only works."""
        return dict(page='managers stuff')

    @expose('luci.templates.index')
    @require(predicates.is_user('editor', msg=l_('Only for the editor')))
    def editor_user_only(self, **kw):
        """Illustrate how a page exclusive for the editor works."""
        return dict(page='editor stuff')


