"""Database Helpers used in luci."""

from luci.model import metadata, DBSession
from luci.model.objects import Node,Cluster,Task
from sqlalchemy.orm.exc import NoResultFound

from luci.lib.ricci_communicator import RicciCommunicator
import luci.lib.ricci_queries as rq

def get_cluster_db_obj(cluster_name):
    db_obj = None
    try:
        db_obj = DBSession.query(Cluster).filter_by(name=cluster_name).one()
    except NoResultFound, e:
        pass
    except Exception, e:
        # log this
        pass
    return db_obj

def get_agent_for_cluster(cluster_name):
    db_obj = get_cluster_db_obj(cluster_name)
    if db_obj is None:
        return None
    node_list = db_obj.nodes
    if len(node_list) < 1:
        return None

    for node_obj in node_list:
        host = node_obj.hostname
        port = node_obj.port
        try:
            rc = RicciCommunicator(host, port=port)
            if rc is not None:
                return rc
        except Exception, e:
            # log this
            pass
    return None

def get_model_for_cluster(cluster_name, rc=None):
    if rc is None:
        rc = get_agent_for_cluster(cluster_name)
        if rc is None:
            return None

    try:
        from luci.lib.ClusterConf.ModelBuilder import ModelBuilder
        conf = rq.getClusterConf(rc)
        if conf is not None:
            model = ModelBuilder(None, conf, rc.os())
            return model
    except Exception, e:
        # log this
        pass

    # Couldn't get the conf from any nodes
    return None

def get_cluster_status(rc):
    try:
        doc = rq.getClusterStatusBatch(rc)
    except Exception, e:
        # log this
        return None
    return doc

def get_cluster_list():
    db_obj = None
    try:
        db_obj = DBSession.query(Cluster).all()
    except NoResultFound, e:
        return None
    except Exception, e:
        # log this
        return None

    return db_obj

def get_cluster_task_status(cluster_name):
    db_obj = get_cluster_db_obj(cluster_name)
    if db_obj is None:
        return None
    tasks = db_obj.tasks
    if len(tasks) > 0:
        pass
        # get status

    return False
