# Copyright (C) 2006-2009 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from ricci_communicator import RicciCommunicator
import threading

class PWorker(threading.Thread):
	# triple := [ host, function, *args ]
	def __init__(self, mutex, ret, host_triples):
		threading.Thread.__init__(self)
		self.mutex = mutex
		self.ret = ret
		self.triples = host_triples

	def run(self):
		while True:
			self.mutex.acquire()
			if len(self.triples) == 0:
				self.mutex.release()
				return
			triple = self.triples.pop()
			self.mutex.release()

			r = { 'ricci': None }

			try:
				rc = RicciCommunicator(triple[0], enforce_trust=False)
				rc.trust()
				r['ricci'] = rc

				if triple[1] is not None:
					if triple[2]:
						args = list(triple[2])
					else:
						args = list()
					args.insert(0, rc)
					r['batch_result'] = triple[1](*args)
			except Exception, e:
				r['error'] = True
				r['err_msg'] = str(e)
				r['ricci'] = None

			self.mutex.acquire()
			self.ret[triple[0]] = r
			self.mutex.release()

class Worker(threading.Thread):
	def __init__(self, mutex, hosts, riccis, func=None, *args):
		threading.Thread.__init__(self)
		self.mutex = mutex
		self.hosts = hosts
		self.riccis = riccis
		self.query_func = func
		self.query_args = args

	def run(self):
		while True:
			self.mutex.acquire()
			if len(self.hosts) == 0:
				self.mutex.release()
				return
			host = self.hosts.pop()
			self.mutex.release()

			r = { 'ricci': None }

			try:
				rc = RicciCommunicator(host, enforce_trust=False)
				rc.trust()
				r['ricci'] = rc
				try:
					r['cluster_name'] = rc.cluster_info()[0]
				except:
					pass

				if self.query_func is not None:
					if self.query_args:
						args = list(self.query_args)
					else:
						args = list()
					args.insert(0, rc)
					r['batch_result'] = self.query_func(*args)
			except Exception, e:
				r['error'] = True
				r['err_msg'] = str(e)
				r['ricci'] = None

			self.mutex.acquire()
			self.riccis[host] = r
			self.mutex.release()

def send_batch_to_hosts(system_list, max_threads, func, *args):
	mutex = threading.RLock()
	threads = list()
	hosts = list()
	num_hosts = 0
	ret = {}

	for host in system_list:
		hosts.append(host)
		num_hosts += 1
		if num_hosts <= max_threads:
			threads.append(Worker(mutex, hosts, ret, func, *args))

	for thread in threads:
		thread.start()
	for thread in threads:
		thread.join()
	return ret

def send_batch_parallel(triples, max_threads):
	mutex = threading.RLock()
	threads = list()
	num_trips = 0
	trips = list()
	ret = {}

	for trip in triples:
		trips.append(trip)
		num_trips += 1
		if num_trips <= max_threads:
			threads.append(PWorker(mutex, ret, triples))

	for thread in threads:
		thread.start()
	for thread in threads:
		thread.join()
	return ret

def get_system_info(self, system_list):
	mutex = threading.RLock()
	hive = []
	ss = {}
	hosts = []

	for system in system_list:
		hosts.append(system[0])
		if len(hosts) < 10:
			hive.append(Worker(mutex, hosts, ss))

	for bee in hive:
		bee.start()

	for bee in hive:
		bee.join()

	for hostname in ss.keys():
		OS = ''
		cluname = ''
		cluali = ''
		key_fp = ''
		trusted = ''
		authed = False
		trusted = False
		ricci = ss[hostname]['ricci']

		if ricci is not None:
			OS = ricci.os()
			cluname = ricci.cluster_info()[0]
			cluali = ricci.cluster_info()[1]
			authed = ricci.authed()
			key_fp = ricci.fingerprint()[1]
			trusted = ricci.trusted()
		else:
			OS = 'System not accessible'

		s = {
			'hostname'	: hostname,
			'os'		: OS,
			'cluname'	: cluname,
			'key_fp'	: key_fp,
			'clualias'	: cluali,
			'available'	: ricci is not None,
			'trusted'	: trusted,
			'authed'	: authed,
			'err_msg'	: ss[hostname].get('err_msg')
		}

		# replace ricci with system's info
		ss[hostname] = s

	ss_list = []
	sorted_keys = ss.keys()
	sorted_keys.sort()

	for name in sorted_keys:
		ss_list.append(ss[name])
	return ss_list
