# -*- coding: utf-8 -*-

from tg import flash, request, redirect, expose, tmpl_context, app_globals, validate
from pylons.i18n import ugettext as _, lazy_ugettext as l_

from luci.controllers.decorators import *

from luci.model import metadata, DBSession
from luci.model.objects import Node, Cluster, Task

from luci.lib.ricci_helpers import send_batch_to_hosts
from luci.lib.ricci_communicator import RicciCommunicator
import luci.lib.ricci_queries as rq

from luci.widgets.create_cluster_form import create_cluster_form

# Imports into module's namespace.
scheme = app_globals.scheme
data = app_globals.data

def validate_create_cluster_form(self, **kw):
    errors = [];

    cluster_name = kw.get('cluster_name')
    if not cluster_name or len(cluster_name) < 1:
        flash(_('No cluster name was given'), 'error')
        return

    if len(cluster_name) > 15:
        flash(_('Cluster names must be less than 16 characters long'), 'error')
        return

    try:
        ret = int(DBSession.query(Cluster).filter_by(name=cluster_name).count())
        if ret != 0:
            flash(_('Luci is already managing a cluster named "%s"') % cluster_name, 'error')
            return
    except Exception, e:
        pass

    create_opt_args = kw.get('opt')
    if create_opt_args is None:
        create_opts = []
    elif isinstance(create_opt_args, list):
        create_opts = create_opt_args
    else:
        create_opts = [ create_opt_args ]

    enable_storage = 'shared_storage' in create_opts
    reboot_nodes = 'reboot_nodes' in create_opts
    download_pkgs = kw.get('download_pkgs') == 'download'

    nodes = []
    cur = 0
    while True:
        cur_host = kw.get('host_list.grow-%d.hostname' % cur)
        cur_passwd = kw.get('host_list.grow-%d.password' % cur)
        cur_port = kw.get('host_list.grow-%d.port' % cur)
        cur += 1
        if cur_port is None:
            break
        if not cur_host:
            continue
        if not cur_passwd:
            errors.append(_('No password was provided for node "%s"') % cur_host)
            continue

        nodes.append([ cur_host, cur_passwd, cur_port ])

    if len(errors) > 0:
        flash(str(errors), 'error')
        return

    cluster_db_obj = Cluster(name=cluster_name,
                            display_name=cluster_name)

    node_list = []
    node_db_obj = {}
    task_db_obj = {}
    for node in nodes:
        try:
            rc = RicciCommunicator(node[0], port=int(node[2]), enforce_trust=False)
            rc.trust()
            rc.auth(node[1])
            if not rc.authed():
                errors.append('Authentication to node %s failed' % node[0])
                break
            else:
                rc = RicciCommunicator(node[0])

            cur_cluster_name = rc.cluster_info()[0]
            if cur_cluster_name:
                errors.append('%s is already a member of a cluster %s' \
                        % (node[0], cur_cluster_name))
                break

            node_list.append(node[0])
            node_db_obj[node[0]] = Node(node_name=node[0],
                                        display_name=node[0],
                                        hostname=node[0],
                                        ipaddr=node[0],
                                        port=node[2])
            DBSession.add(node_db_obj[node[0]])
        except Exception, e:
            errors.append(str(e))

    if len(errors) > 0:
        flash(_('The following errors occurred: %s') % str(errors), 'error')
        DBSession.rollback()
        return

    ret = send_batch_to_hosts(node_list, 10, rq.create_cluster,
                'rhel5', cluster_name, cluster_name,
                node_list, True, True, enable_storage, False,
                download_pkgs, None, reboot_nodes)

    for i in ret.iterkeys():
        if ret[i].has_key('error'):
           errors.append(_('Unable to connect to the ricci agent on node %s: %s') % (i, ret[i]['err_msg']))
        task_id = ret[i]['batch_result'][0]
        task_obj = Task(batch_id=task_id,
                        batch_status=0,
						redirect_url='/clusters/%s/' % cluster_name,
                        status_msg=_('Creating node "%s" for cluster "%s"') % (i, cluster_name))
        DBSession.add(task_obj)
        task_db_obj[i] = task_obj

    try:
        cluster_db_obj.nodes = node_db_obj.values()
        cluster_db_obj.tasks = task_db_obj.values()
        for i in node_db_obj.iterkeys():
            node_db_obj[i].tasks = [ task_db_obj[i] ]
        DBSession.add(cluster_db_obj)
    except Exception, e:
        DBSession.rollback()
        flash(_('An error occurred while updating the luci database: %s') % str(e), 'error')
        return

    flash(_('Creating the cluster "%s"... Please wait...') % cluster_name, 'info')
    redirect("/clusters/%s/" % cluster_name)
